import numpy as np

from .calculations import centerOfMass, angle, distance


def generateObjects(count, center=0, width=100000, massmult=10, vmult=100, include_blackhole=False):
    '''Generate objects in a 2d normal distribution (to simulate a galaxy) with initial velocities
    roughly orthogonal to the center of mass in a e.g. rotating

    Args:
        count (int): number of objects to generate
        center (int, optional): x/y center to use. Defaults to 0.
        width (int, optional): how wide to generate x/y across. Defaults to 100000.
        massmult (int, optional): mass multiplier for generated objects. Defaults to 10.
        vmult (int, optional): velocity multiplier for generated objects. Defaults to 100.
        include_blackhole (bool): put a large mass in the center

    Returns:
        a (`count`, 5) list/array of positions

        The return value should look like:
            [[x1, y1, mass1, velocityx1, velocityy1],
             [x2, y2, mass2, velocityx2, velocityy2],
             ...
            ]
    '''
    # first, generate the position and mass distribution
    positions = np.random.multivariate_normal(
        [center, center],
        [
            [width, 0],
            [0, width],
        ],
        count,
    )
    weights = np.random.random((count, 1)) * massmult
    data = np.concatenate((positions, weights), axis=1)

    # now calculate the center of mass
    com = centerOfMass(data)

    # now get the angles between the com and the data points
    angles = [angle(com, datum[:2]) for datum in data]

    # if each point is θ from com, then 90+θ (π/2+θ rad) should be good
    angles = angles + np.ones(count) * np.pi / 2
    velocity_magnitudes = np.array([vmult * np.pi * (distance(com, vec[:2]) / (massmult * width) ) for vec in data])
    velocities = np.array(
        [
            [np.cos(theta) * mag, np.sin(theta) * mag]
            for theta, mag in zip(angles, velocity_magnitudes)
        ]
    )

    # concat velocities to data
    data = np.concatenate((data, velocities), axis=1)

    if include_blackhole:
        # as a last step, put a stationary black hole at the center
        data[0] = np.array([center, center, data[:, 2].max() * 10000, 0, 0])

    return data
